import java.awt.Color;
import java.net.MalformedURLException;
import java.net.URL;

import javax.swing.JApplet;
import javax.swing.JFrame;

import rubine.Classifier;
import rubine.ClassifierNotTrainedException;
import rubine.Gesture;
import rubine.GestureClass;
import sm.SMCanvas;
import sm.SMPolyLine;
import sm.SMShape;
import sm.SMVirtualEvent;
import sm.StateMachine;

public class CutCopyPaste extends JApplet {
	
	StateMachine smGesture;
	StateMachine smInk;
	SMCanvas canvas = new SMCanvas(300, 300);
	
	
	public void init() {
		try {
			javax.swing.SwingUtilities.invokeAndWait(new Runnable() {
				public void run() {
					createGUI();
				}
			});
		} catch (Exception e) {
			e.printStackTrace();
			System.err.println("createGUI didn't successfully complete");
		}
	}
	
	public void createGUI() {
		setSize(300, 300);
		getContentPane().add(canvas);
		
		smGesture = new StateMachine("gesture", canvas) {
			
			SMShape shapeCopied = null;
			SMShape shapeCut = null;
			
			public State start = new State("start") {
				
				Transition create = new Click(BUTTON1) {
					public void action() {
						canvas.newRectangle(getPoint().getX(), getPoint().getY(), 40, 30).setFillPaint(new Color((int)(Math.random()*255), (int)(Math.random()*255), (int)(Math.random()*255)));
					}
				};
				
				Transition copy = new EventOnShape("copy") {
					public void action() {
						shapeCopied = getShape();
						shapeCut = null;
					}
				};
				
				Transition cut = new EventOnShape("cut"){
					public void action(){
						shapeCut = getShape();
						shapeCut.addGhost();
						shapeCut.setDrawable(false);
						shapeCopied = null;
					}
				};
				
				Transition paste = new EventOnPosition("paste"){
					public void action(){
						if(shapeCopied != null) {
							SMShape newShape = new SMShape();
							shapeCopied.copyTo(newShape);
							newShape.translateBy(getPoint().getX() - shapeCopied.getCenterX(), getPoint().getY() - shapeCopied.getCenterY());
						} else {
							if(shapeCut != null) {
								shapeCut.translateBy(getPoint().getX() - shapeCut.getCenterX(), getPoint().getY() - shapeCut.getCenterY());
								shapeCut.removeGhost();
								shapeCut.setDrawable(true);
								shapeCopied = shapeCut;
							}
						}
						
					}
				};
			};
		};		
		
		smInk = new StateMachine("ink", canvas) {
			
			Gesture gesture;
			SMPolyLine ink = (SMPolyLine) canvas.newPolyLine(0, 0).setPickable(false).setFilled(false);
			Classifier classifier;
			
			public void doReset(){
				try {
					classifier = Classifier.newClassifier(new URL("http://www.lri.fr/~appert/SMCanvas/CutCopyPaste/cutcopypaste.cl"));
					System.out.println(new URL("http://www.lri.fr/~appert/SMCanvas/CutCopyPaste/cutcopypaste.cl"));
				} catch (MalformedURLException e) {
					e.printStackTrace();
				}
				gesture = new Gesture();
			}
			
			public State start = new State("start") {
				
				Transition begin = new Press(BUTTON1){
					public void action(){
						ink.reset(getPoint().getX(), getPoint().getY()).setDrawable(true).aboveAll();
						gesture.reset();
						gesture.addPoint(getPoint().getX(), getPoint().getY());
					}
				};
				
				Transition draw = new Drag(BUTTON1){
					public void action(){
						ink.lineTo(getPoint().getX(), getPoint().getY());
						gesture.addPoint(getPoint().getX(), getPoint().getY());
					}
				};
				
				Transition end = new Release(BUTTON1){
					public void action(){
						armTimer(500);
						gesture.addPoint(getPoint().getX(), getPoint().getY());
						try {
							GestureClass gc = classifier.classify(gesture);
							if(gc != null) smGesture.processEvent(new SMVirtualEvent(gc.getName()), getModifier(), getPoint());
						} catch (ClassifierNotTrainedException e) {
							e.printStackTrace();
						}
					}
				};
				
				Transition timeout = new Transition("TimeOut") {
					public void action(){
						ink.setDrawable(false);
					}
				};
			};
		};		
	}
	
	public static void main(String[] args) {
		new CutCopyPaste();
	}
	
	
	
}
